/* voicectl.c
 * Michael McTernan, mm7323@bris.ac.uk
 * 1/10/00
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <syslog.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <ctype.h>

#include "IniReader.h"
#include "Modem.h"

/* The menu structures */
typedef struct {
  char   *dtmfCode;
  char   *action; 
} menu_entry_t;

typedef struct {
  int           nOpts;
  menu_entry_t  menuOpts[6];
} menu_t;

int    theMenuN;
menu_t theMenu[10];

int dtmfInitialTimeout =6;
int dtmfInterkeyTimeout=2;

/* Prototypes */
static void iniReadOptions();
static void renderMenus();
static int  processAction(const char *action);
static void child();


/*main*/
int main(int argc,char *argv[]) {
  pid_t p;

  /* Read the conf file options */
  iniReadOptions();

  /* Crude command line parsing */
  if(argc>1)
    modemDevice=argv[1];

  if(argc>2 && strcmp(argv[2],"-d"))
    debugFlags=3;

  /* Background */
  if(fork()!=0)
    exit(0);

  while(1) {
    if((p=fork())==0)
      child();

    waitpid(p,NULL,0);
  }
}


/*child*/
/* This does all the work
 */
static void child() {

  openlog("telephonectl",LOG_CONS | LOG_PID,LOG_DAEMON);
  syslog(LOG_INFO,"Starting...");

  if(!modemOpen()) {
    syslog(LOG_INFO,"Failed to open modem: %m");
    syslog(LOG_INFO,"Sleeping for 60 seconds before retry");
    sleep(60);
    exit(0);
  }

  modemInit();

  modemSendExpect("AT#CLS=8","OK");     /* Call mode */

  /* Wait for a ring */
  syslog(LOG_INFO,"Waiting for RING");
  modemExpect("RING");
  syslog(LOG_INFO,"RING");

  /* Answer phone */
  modemSendExpect("ATA","VCON");
  modemSendExpect("AT#VLS=4","OK");     /* Voice Line Select */
  modemSendExpect("AT#VSP=255","OK");   /* Voice Silence Period */
  modemSendExpect("AT#VTD=1,1,1","OK"); /* DTMF options */

  renderMenus();

  closelog();
}


/*renderMenus*/
/* Simply takes the user through the menu options
 */
static void renderMenus() {
  int  curMenu=0;
  char DTMF[11];
  int  DTMFlen=0;
  int  t,l;
  int  timeout;
  char c;


  /* Give the user 5 attempts at each menu */
  for(l=0;l<5 && curMenu>=0;l++) {

    /* Play the menu sample */
    modemSendExpect("AT#VTS=[900,0,1]","OK");

    /* Now wait for all the DTMF keys */
    DTMF[DTMFlen=0]='\0';
   
    timeout=dtmfInitialTimeout;
    while((c=modemReadDLE(timeout,0))!='\0' && DTMFlen<10) {
      if(isdigit(c)) {
        DTMF[DTMFlen++]=c;
        timeout=dtmfInterkeyTimeout;
      }
    }

    /* Low beep */
    modemSendExpect("AT#VTS=[200,0,1]","OK");

    DTMF[DTMFlen]='\0';
    syslog(LOG_INFO,"Read DTMF %s",DTMF);

    /* Find which sub menu to goto */
    for(t=0;t<theMenu[curMenu].nOpts;t++) {

      if(theMenu[curMenu].menuOpts[t].dtmfCode!=NULL && 
           strcmp(theMenu[curMenu].menuOpts[t].dtmfCode,DTMF)==0) {

        modemSendExpect("AT#VTS=[2000,500,10]","OK");
        curMenu=processAction(theMenu[curMenu].menuOpts[t].action);        
        l=0;
      }

    }

  }

  exit(0);
}


/*processAction*/
/*
 */
static int processAction(const char *action) {
  int r=-1;

  if(action==NULL) {
    syslog(LOG_ERR,"No action defined for selected menu option.");
    syslog(LOG_ERR,"Hanging.");
    return -1;
  }

  if(*action=='*') {
    action++;

    if(strncasecmp(action,"hangup",6)==0) {
      syslog(LOG_INFO,"Closing modem.");
      modemClose();
      if((action=strstr(action,"#"))!=NULL) {
        syslog(LOG_INFO,"Sleeping for 5 seconds.");
        sleep(5);
        action++;
        syslog(LOG_INFO,"Calling: %s",action);
        system(action);
      }
      r=-1;
    }

    if(strncasecmp(action,"menu",4)==0) {
      sscanf(action,"menu%d",&r);
    }
  }    
  
  if((action=strstr(action,"#"))!=NULL) {
    action++;
    syslog(LOG_INFO,"Calling: %s",action);
    system(action);
  }

  return r;
}

/*iniReadOptions*/
/* Reads the options using the table driven
 *  IniReader module.
 */
static void iniReadOptions() {
  int   curMenu;
  ini_t iniTable[]={{"Parity","%d",&paritySetting,0,NULL},
                    {"FlowControl","%d",&flowControl,0,NULL},
                    {"Bits","%d",&dataBits,0,NULL},
                    {"StopBits","%d",&stopBits,0,NULL},
                    {"DebugFlags","%d",&debugFlags,0,NULL},
                    {"ModemDevice","%as",&modemDevice,0,NULL},
                   
                    {"MenuCount","%d",&theMenuN,0,NULL},
                    {"Menu","%d",&curMenu,0,NULL},
 
                    {"MenuOptCount","%d",&theMenu->nOpts,sizeof(menu_t),&curMenu},

                    {"DTMF0","%as",&theMenu->menuOpts[0].dtmfCode,sizeof(menu_t),&curMenu},
                    {"DTMF1","%as",&theMenu->menuOpts[1].dtmfCode,sizeof(menu_t),&curMenu},
                    {"DTMF2","%as",&theMenu->menuOpts[2].dtmfCode,sizeof(menu_t),&curMenu},
                    {"DTMF3","%as",&theMenu->menuOpts[3].dtmfCode,sizeof(menu_t),&curMenu},
                    {"DTMF4","%as",&theMenu->menuOpts[4].dtmfCode,sizeof(menu_t),&curMenu},
                    {"DTMF5","%as",&theMenu->menuOpts[5].dtmfCode,sizeof(menu_t),&curMenu},

                    {"Action0","%as",&theMenu->menuOpts[0].action,sizeof(menu_t),&curMenu},
                    {"Action1","%as",&theMenu->menuOpts[1].action,sizeof(menu_t),&curMenu},
                    {"Action2","%as",&theMenu->menuOpts[2].action,sizeof(menu_t),&curMenu},
                    {"Action3","%as",&theMenu->menuOpts[3].action,sizeof(menu_t),&curMenu},
                    {"Action4","%as",&theMenu->menuOpts[4].action,sizeof(menu_t),&curMenu},
                    {"Action5","%as",&theMenu->menuOpts[5].action,sizeof(menu_t),&curMenu},

                    {NULL,NULL,NULL,0,NULL}
                   };

  INI_ReadFile(iniTable,"/etc/telephonectld.conf");

}



